@@ -480,8 +480,19 @@ class TaskExecutor:
             raise self._loop_eval_error  # pylint: disable=raising-bad-type
 
         # if we ran into an error while setting up the PlayContext, raise it now, unless is known issue with delegation
-        if context_validation_error is not None and not (self._task.delegate_to and isinstance(context_validation_error, AnsibleUndefinedVariable)):
-            raise context_validation_error  # pylint: disable=raising-bad-type
+        # and undefined vars (correct values are in cvars later on and connection plugins, if still error, blows up there)
+        if context_validation_error is not None:
+            raiseit = True
+            if self._task.delegate_to:
+                if isinstance(context_validation_error, AnsibleUndefinedVariable):
+                    raiseit = False
+                elif isinstance(context_validation_error, AnsibleParserError):
+                    # parser error, might be cause by undef too
+                    orig_exc = getattr(context_validation_error, 'orig_exc', None)
+                    if isinstance(orig_exc, AnsibleUndefinedVariable):
+                        raiseit = False
+            if raiseit:
+                raise context_validation_error  # pylint: disable=raising-bad-type
 
         # if this task is a TaskInclude, we just return now with a success code so the
         # main thread can expand the task list for the given host
@@ -502,7 +513,7 @@ class TaskExecutor:
         # Now we do final validation on the task, which sets all fields to their final values.
         try:
             self._task.post_validate(templar=templar)
-        except AnsibleError:
+        except AnsibleError as e:
             raise
         except Exception:
             return dict(changed=False, failed=True, _ansible_no_log=self._play_context.no_log, exception=to_text(traceback.format_exc()))
@@ -538,12 +549,19 @@ class TaskExecutor:
         plugin_vars = self._set_connection_options(cvars, templar)
         templar.available_variables = orig_vars
 
+        # TODO: eventually remove this block as this should be a 'consequence' of 'forced_local' modules
+        # special handling for python interpreter for network_os, default to ansible python unless overriden
+        if 'ansible_network_os' in cvars and 'ansible_python_interpreter' not in cvars:
+            # this also avoids 'python discovery'
+            cvars['ansible_python_interpreter'] = sys.executable
+
         # get handler
         self._handler = self._get_action_handler(connection=self._connection, templar=templar)
 
         # Apply default params for action/module, if present
         self._task.args = get_action_args_with_defaults(
-            self._task.action, self._task.args, self._task.module_defaults, templar, self._task._ansible_internal_redirect_list
+            self._task.resolved_action, self._task.args, self._task.module_defaults, templar,
+            action_groups=self._task._parent._play._action_groups
         )
 
         # And filter out any fields which were set to default(omit), and got the omit token value
@@ -580,10 +598,8 @@ class TaskExecutor:
                     old_sig = signal.signal(signal.SIGALRM, task_timeout)
                     signal.alarm(self._task.timeout)
                 result = self._handler.run(task_vars=variables)
-            except AnsibleActionSkip as e:
-                return dict(skipped=True, msg=to_text(e))
-            except AnsibleActionFail as e:
-                return dict(failed=True, msg=to_text(e))
+            except (AnsibleActionFail, AnsibleActionSkip) as e:
+                return e.result
             except AnsibleConnectionFailure as e:
                 return dict(unreachable=True, msg=to_text(e))
             except TaskTimeoutError as e:
@@ -599,17 +615,34 @@ class TaskExecutor:
             # preserve no log
             result["_ansible_no_log"] = self._play_context.no_log
 
+            if self._task.action not in C._ACTION_WITH_CLEAN_FACTS:
+                result = wrap_var(result)
+
             # update the local copy of vars with the registered value, if specified,
             # or any facts which may have been generated by the module execution
             if self._task.register:
                 if not isidentifier(self._task.register):
                     raise AnsibleError("Invalid variable name in 'register' specified: '%s'" % self._task.register)
 
-                vars_copy[self._task.register] = result = wrap_var(result)
+                vars_copy[self._task.register] = result
 
             if self._task.async_val > 0:
                 if self._task.poll > 0 and not result.get('skipped') and not result.get('failed'):
                     result = self._poll_async_result(result=result, templar=templar, task_vars=vars_copy)
+                    if result.get('failed'):
+                        self._final_q.send_callback(
+                            'v2_runner_on_async_failed',
+                            TaskResult(self._host.name,
+                                       self._task._uuid,
+                                       result,
+                                       task_fields=self._task.dump_attrs()))
+                    else:
+                        self._final_q.send_callback(
+                            'v2_runner_on_async_ok',
+                            TaskResult(self._host.name,
+                                       self._task._uuid,
+                                       result,
+                                       task_fields=self._task.dump_attrs()))
 
                 # ensure no log is preserved
                 result["_ansible_no_log"] = self._play_context.no_log
@@ -633,7 +666,13 @@ class TaskExecutor:
 
             if 'ansible_facts' in result and self._task.action not in C._ACTION_DEBUG:
                 if self._task.action in C._ACTION_WITH_CLEAN_FACTS:
-                    vars_copy.update(result['ansible_facts'])
+                    if self._task.delegate_to and self._task.delegate_facts:
+                        if '_ansible_delegated_vars' in vars_copy:
+                            vars_copy['_ansible_delegated_vars'].update(result['ansible_facts'])
+                        else:
+                            vars_copy['_ansible_delegated_vars'] = result['ansible_facts']
+                    else:
+                        vars_copy.update(result['ansible_facts'])
                 else:
                     # TODO: cleaning of facts should eventually become part of taskresults instead of vars
                     af = wrap_var(result['ansible_facts'])
@@ -657,18 +696,27 @@ class TaskExecutor:
             if 'changed' not in result:
                 result['changed'] = False
 
+            if self._task.action not in C._ACTION_WITH_CLEAN_FACTS:
+                result = wrap_var(result)
+
             # re-update the local copy of vars with the registered value, if specified,
             # or any facts which may have been generated by the module execution
             # This gives changed/failed_when access to additional recently modified
             # attributes of result
             if self._task.register:
-                vars_copy[self._task.register] = result = wrap_var(result)
+                vars_copy[self._task.register] = result
 
             # if we didn't skip this task, use the helpers to evaluate the changed/
             # failed_when properties
             if 'skipped' not in result:
-                _evaluate_changed_when_result(result)
-                _evaluate_failed_when_result(result)
+                try:
+                    condname = 'changed'
+                    _evaluate_changed_when_result(result)
+                    condname = 'failed'
+                    _evaluate_failed_when_result(result)
+                except AnsibleError as e:
+                    result['failed'] = True
+                    result['%s_when_result' % condname] = to_text(e)
 
             if retries > 1:
                 cond = Conditional(loader=self._loader)
@@ -698,10 +746,13 @@ class TaskExecutor:
                 result['attempts'] = retries - 1
                 result['failed'] = True
 
+        if self._task.action not in C._ACTION_WITH_CLEAN_FACTS:
+            result = wrap_var(result)
+
         # do the final update of the local variables here, for both registered
         # values and any facts which may have been created
         if self._task.register:
-            variables[self._task.register] = result = wrap_var(result)
+            variables[self._task.register] = result
 
         if 'ansible_facts' in result and self._task.action not in C._ACTION_DEBUG:
             if self._task.action in C._ACTION_WITH_CLEAN_FACTS:
@@ -727,6 +778,7 @@ class TaskExecutor:
             for k in plugin_vars:
                 result["_ansible_delegated_vars"][k] = cvars.get(k)
 
+            # note: here for callbacks that rely on this info to display delegation
             for requireshed in ('ansible_host', 'ansible_port', 'ansible_user', 'ansible_connection'):
                 if requireshed not in result["_ansible_delegated_vars"] and requireshed in cvars:
                     result["_ansible_delegated_vars"][requireshed] = cvars.get(requireshed)
@@ -751,7 +803,7 @@ class TaskExecutor:
         # that (with a sleep for "poll" seconds between each retry) until the
         # async time limit is exceeded.
 
-        async_task = Task().load(dict(action='async_status jid=%s' % async_jid, environment=self._task.environment))
+        async_task = Task.load(dict(action='async_status', args={'jid': async_jid}, environment=self._task.environment))
 
         # FIXME: this is no longer the case, normal takes care of all, see if this can just be generalized
         # Because this is an async task, the action handler is async. However,
@@ -803,21 +855,21 @@ class TaskExecutor:
                     'v2_runner_on_async_poll',
                     TaskResult(
                         self._host.name,
-                        async_task,  # We send the full task here, because the controller knows nothing about it, the TE created it
+                        async_task._uuid,
                         async_result,
-                        task_fields=self._task.dump_attrs(),
+                        task_fields=async_task.dump_attrs(),
                     ),
                 )
 
         if int(async_result.get('finished', 0)) != 1:
             if async_result.get('_ansible_parsed'):
-                return dict(failed=True, msg="async task did not complete within the requested time - %ss" % self._task.async_val)
+                return dict(failed=True, msg="async task did not complete within the requested time - %ss" % self._task.async_val, async_result=async_result)
             else:
                 return dict(failed=True, msg="async task produced unparseable results", async_result=async_result)
         else:
             # If the async task finished, automatically cleanup the temporary
             # status file left behind.
-            cleanup_task = Task().load(
+            cleanup_task = Task.load(
                 {
                     'async_status': {
                         'jid': async_jid,
