@@ -20,6 +20,7 @@ DOCUMENTATION = """
         description:
             - Whether to convert YAML into data. If False, strings that are YAML will be left untouched.
             - Mutually exclusive with the jinja2_native option.
+        default: true
       variable_start_string:
         description: The string marking the beginning of a print statement.
         default: '{{'
@@ -40,6 +41,19 @@ DOCUMENTATION = """
         default: False
         version_added: '2.11'
         type: bool
+      template_vars:
+        description: A dictionary, the keys become additional variables available for templating.
+        default: {}
+        version_added: '2.3'
+        type: dict
+      comment_start_string:
+        description: The string marking the beginning of a comment statement.
+        version_added: '2.12'
+        type: str
+      comment_end_string:
+        description: The string marking the end of a comment statement.
+        version_added: '2.12'
+        type: str
 """
 
 EXAMPLES = """
@@ -50,6 +64,10 @@ EXAMPLES = """
 - name: show templating results with different variable start and end string
   debug:
     msg: "{{ lookup('template', './some_template.j2', variable_start_string='[%', variable_end_string='%]') }}"
+
+- name: show templating results with different comment start and end string
+  debug:
+    msg: "{{ lookup('template', './some_template.j2', comment_start_string='[#', comment_end_string='#]') }}"
 """
 
 RETURN = """
@@ -78,13 +96,19 @@ display = Display()
 class LookupModule(LookupBase):
 
     def run(self, terms, variables, **kwargs):
-        convert_data_p = kwargs.get('convert_data', True)
-        lookup_template_vars = kwargs.get('template_vars', {})
-        jinja2_native = kwargs.get('jinja2_native', False)
+
         ret = []
 
-        variable_start_string = kwargs.get('variable_start_string', None)
-        variable_end_string = kwargs.get('variable_end_string', None)
+        self.set_options(var_options=variables, direct=kwargs)
+
+        # capture options
+        convert_data_p = self.get_option('convert_data')
+        lookup_template_vars = self.get_option('template_vars')
+        jinja2_native = self.get_option('jinja2_native')
+        variable_start_string = self.get_option('variable_start_string')
+        variable_end_string = self.get_option('variable_end_string')
+        comment_start_string = self.get_option('comment_start_string')
+        comment_end_string = self.get_option('comment_end_string')
 
         if USE_JINJA2_NATIVE and not jinja2_native:
             templar = self._templar.copy_with_new_env(environment_class=AnsibleEnvironment)
@@ -123,6 +147,8 @@ class LookupModule(LookupBase):
 
                 with templar.set_temporary_context(variable_start_string=variable_start_string,
                                                    variable_end_string=variable_end_string,
+                                                   comment_start_string=comment_start_string,
+                                                   comment_end_string=comment_end_string,
                                                    available_variables=vars, searchpath=searchpath):
                     res = templar.template(template_data, preserve_trailing_newlines=True,
                                            convert_data=convert_data_p, escape_backslashes=False)
